using System;
using System.Collections.Generic;
using System.Timers;
using AntsAI_bot_Surya1;
using AntBot.Algorithms;
using AStarAlgo;
using System.Diagnostics;

namespace Ants
{
    public class GameState
    {
        public static bool[,] m_mapWater = null; //Water map (saved)
        public static int[,] m_mapVisible = null; //number of my ants seeing a specific Tile
        public static int[,] m_enemyAntMap = null; 
        public static int[,] m_mapLastTurnSeen = null; //Map of last turn value I've seen the specific Tile
		public static bool[,] m_containsMyAnt = null; //Map containing my ants (updated if they move in real time)

		//Fighting "magic" arrays (thx memetix for the explanation)
        public static int[,] m_fightTotalArray = null;
        public static int[,,] m_fightInfluenceArray = null;
        public static int[,,] m_fightFightingArray = null;
        public static eFightingStatus[, ,] m_fightStatusArray = null;

		//Pathfinder
        public static PathFinderFast myPathFinder = null;

        public static int g_tourNb = 0;
        public static int g_EnemyPlayerNumber = 0;

        public static int Width { get; private set; }
        public static int Height { get; private set; }

        public int LoadTime { get; private set; }
        public static int TurnTime { get; private set; }

        public static List<Location> g_VisibleOffsets = new List<Location>();
        public static List<Location> g_Attackoffsets = new List<Location>();
        public static List<Location> g_AttackoffsetsStrict = new List<Location>();

        public static List<Location> g_AttackoffsetsStrictIfTopFree = new List<Location>();
        public static List<Location> g_AttackoffsetsStrictIfBotFree = new List<Location>();
        public static List<Location> g_AttackoffsetsStrictIfLeftFree = new List<Location>();
        public static List<Location> g_AttackoffsetsStrictIfRightFree = new List<Location>();


        public static Stopwatch TIMER_TIMEOUT = new Stopwatch();
        public static int iMillisecBeforeGlobalTimeout = -1;
        public static int iMillisecBeforeComputeTimeout = -1;


        private static DateTime turnStart;
        public int TimeRemaining
        {
            get
            {
                TimeSpan timeSpent = DateTime.Now - turnStart;
                return TurnTime - timeSpent.Milliseconds;
            }
        }

        public static bool bDistanceModifiedThisTurn = false;
        public static bool IsTimeOutGlobal()
        {
            if (TIMER_TIMEOUT.ElapsedMilliseconds > iMillisecBeforeGlobalTimeout)
            {
                if (!bDistanceModifiedThisTurn && MyBot.g_iDistanceAllowed > MyBot.g_iDistanceAllowedMin)
                {
                    bDistanceModifiedThisTurn = true;
                    MyBot.g_iDistanceAllowed--;
                    #region log
#if LOG_ENABLED
                Log(string.Format("*TIMEOUT : g_iDistanceAllowed decreased to {0} **", g_iDistanceAllowed), 100);
#endif
                    #endregion
                }
                return true;
            }
            else
                return false;
        }

        public static bool IsTimeOutCompute()
        {
            if (TIMER_TIMEOUT.ElapsedMilliseconds > iMillisecBeforeComputeTimeout)
            {
                if (!bDistanceModifiedThisTurn && MyBot.g_iDistanceAllowed > MyBot.g_iDistanceAllowedMin)
                {
                    bDistanceModifiedThisTurn = true;
                    MyBot.g_iDistanceAllowed--;
                    #region log
#if LOG_ENABLED
                Log(string.Format("*TIMEOUT : g_iDistanceAllowed decreased to {0} **", g_iDistanceAllowed), 100);
#endif
                    #endregion
                }
                return true;
            }
            else
                return false;
        }

        public int ViewRadius2 { get; private set; }
        public int ViewRadius { get; private set; }
        public int AttackRadius2 { get; private set; }
        public int AttackRadius { get; private set; }
        public int SpawnRadius2 { get; private set; }

        public List<Ant> MyAnts { get; private set; }
		public static List<Ant> FreeAnts { get; private set; }
        public List<AntHill> MyHills { get; private set; }
        public List<Ant> EnemyAnts { get; private set; }
        public List<AntHill> EnemyHills { get; private set; }
        public List<Location> DeadTiles { get; private set; }
        public List<Location> FoodTiles { get; private set; }

        public Tile this[Location location]
        {
            get { return this.map[location.Row, location.Col]; }
        }

        public Tile this[int row, int col]
        {
            get { return this.map[row, col]; }
        }

        private Tile[,] map;

        public GameState(int width, int height,
                                            int turntime, int loadtime,
                                            int viewradius2, int attackradius2, int spawnradius2)
        {

            Width = width;
            Height = height;

            LoadTime = loadtime;
            TurnTime = turntime;

            ViewRadius2 = viewradius2;
            ViewRadius = (int)Math.Floor(Math.Sqrt(this.ViewRadius2));
            AttackRadius2 = attackradius2;
            AttackRadius = (int)Math.Floor(Math.Sqrt(this.AttackRadius2));
            SpawnRadius2 = spawnradius2;

            MyAnts = new List<Ant>();
						FreeAnts = new List<Ant>();
            MyHills = new List<AntHill>();
            EnemyAnts = new List<Ant>();
            EnemyHills = new List<AntHill>();
            DeadTiles = new List<Location>();
            FoodTiles = new List<Location>();

            map = new Tile[height, width];
            for (int row = 0; row < height; row++)
            {
                for (int col = 0; col < width; col++)
                {
                    map[row, col] = Tile.Land;
                }
            }

            CreateMap();
            CreateOffset();
        }

        private void CreateOffset()
        {
            for (int r = -1 * ViewRadius; r <= ViewRadius; ++r)
            {
                for (int c = -1 * ViewRadius; c <= ViewRadius; ++c)
                {
                    int square = r * r + c * c;
                    if (square < ViewRadius2)
                    {
                        g_VisibleOffsets.Add(new Location(r, c));
                    }
                }
            }

            //En dur for now
            //*int AttackRadiusP1 = AttackRadius + 1;
            //int AttackRadius2P1 = AttackRadiusP1 * AttackRadiusP1;
            for (int r = -1 * AttackRadius; r <= AttackRadius; ++r)
            {
                for (int c = -1 * AttackRadius; c <= AttackRadius; ++c)
                {
                    int square = r * r + c * c;
                    if (square < AttackRadius2)
                    {
                        g_AttackoffsetsStrict.Add(new Location(r, c));
                        g_Attackoffsets.Add(new Location(r, c));
                    }
                }
            }
            //A la main pr finir. Surya's touch :D Histoire que a soit pas gnrique. Faut pas non plus dconner. [TODO!]
            g_Attackoffsets.Add(new Location(-3, -1));
            g_Attackoffsets.Add(new Location(-3, 0));
            g_Attackoffsets.Add(new Location(-3, 1));
            g_Attackoffsets.Add(new Location(-2, -2));
            g_Attackoffsets.Add(new Location(-2, -1));
            g_Attackoffsets.Add(new Location(-2, 1));
            g_Attackoffsets.Add(new Location(-2, 2));
            g_Attackoffsets.Add(new Location(-1, -3));
            g_Attackoffsets.Add(new Location(-1, -2));
            g_Attackoffsets.Add(new Location(-1, 2));
            g_Attackoffsets.Add(new Location(-1, 3));
            g_Attackoffsets.Add(new Location(0, -3));

            g_Attackoffsets.Add(new Location(0, +3));
            g_Attackoffsets.Add(new Location(1, 3));
            g_Attackoffsets.Add(new Location(1, 2));
            g_Attackoffsets.Add(new Location(1, -2));
            g_Attackoffsets.Add(new Location(1, -3));
            g_Attackoffsets.Add(new Location(2, 1));
            g_Attackoffsets.Add(new Location(2, 2));
            g_Attackoffsets.Add(new Location(2, -1));
            g_Attackoffsets.Add(new Location(2, -2));
            g_Attackoffsets.Add(new Location(3, 1));
            g_Attackoffsets.Add(new Location(3, 0));
            g_Attackoffsets.Add(new Location(3, -1));



        }

        #region State mutators
        public void StartNewTurn()
        {
            g_tourNb++;
            // start timer
            turnStart = DateTime.Now;
            TIMER_TIMEOUT.Reset();
            TIMER_TIMEOUT.Start();
            bDistanceModifiedThisTurn = false;

            m_enemyAntMap = new int[Width, Height];
			m_containsMyAnt = new bool[Width, Height];

            // clear ant data
            foreach (Location loc in MyAnts) map[loc.Row, loc.Col] = Tile.Land;
            foreach (Location loc in MyHills) map[loc.Row, loc.Col] = Tile.Land;
            foreach (Location loc in EnemyAnts) map[loc.Row, loc.Col] = Tile.Land;
            foreach (Location loc in EnemyHills) map[loc.Row, loc.Col] = Tile.Land;
            foreach (Location loc in DeadTiles) map[loc.Row, loc.Col] = Tile.Land;

            MyHills.Clear();
            MyAnts.Clear();
						FreeAnts.Clear();
            EnemyHills.Clear();
            EnemyAnts.Clear();
            DeadTiles.Clear();

            // set all known food to unseen
            foreach (Location loc in FoodTiles) map[loc.Row, loc.Col] = Tile.Land;
            FoodTiles.Clear();
        }

        public void AddAnt(int row, int col, int team)
        {
            if (g_EnemyPlayerNumber < team)
                g_EnemyPlayerNumber = team;

            map[row, col] = Tile.Ant;

            Ant ant = new Ant(row, col, team);
            if (team == 0)
            {
                MyAnts.Add(ant);
								FreeAnts.Add(ant);
								m_containsMyAnt[col, row] = true;
            }
            else
            {
                EnemyAnts.Add(ant);
                m_enemyAntMap[col, row] = team;
            }
        }

        public static void CreateMap()
        {
            int iHeight = Height;
            int iWidth = Width;
            bool[,] map = new bool[iWidth, iHeight];
            byte[,] mapWaterSaved = new byte[iHeight, iWidth];
            AntHill anthill = new AntHill(0, 0, 1);

            if (m_mapWater == null)
                m_mapWater = new bool[iWidth, iHeight];

                m_mapVisible = new int[iWidth, iHeight];

            if (m_mapLastTurnSeen == null)
                m_mapLastTurnSeen = new int[iWidth, iHeight];



            for (int row = 0; row < iHeight; row++)
            {
                for (int col = 0; col < iWidth; col++)
                {
                    mapWaterSaved[row, col] = 1;
                    m_mapWater[col, row] = false;
                    map[col, row] = false;
                    m_mapLastTurnSeen[col, row] = -1;
                }
            }
            myPathFinder = new PathFinderFast(mapWaterSaved);
        }

        public void UpdateVisibleMap()
        {
            m_mapVisible = new int[Width, Height];

            foreach (Ant ant in MyAnts)
            {
                foreach (Location offset in g_VisibleOffsets)
                {
                    int colMaybe = ant.Col + offset.Col;
                    if (colMaybe < 0)
                        colMaybe += Width;
                    else if (colMaybe >= Width)
                        colMaybe -= Width;

                    int rowMaybe = ant.Row + offset.Row;
                    if (rowMaybe < 0)
                        rowMaybe += Height;
                    else if (rowMaybe >= Height)
                        rowMaybe -= Height;

                    GameState.m_mapVisible[colMaybe, rowMaybe] += 1;
                    m_mapLastTurnSeen[colMaybe, rowMaybe] = g_tourNb;
                }
            }
        }

        public void UpdateVisibleMap(Location previousloc, Location newLoc)
        {
            foreach (Location offset in g_VisibleOffsets)
            {
                int colPrevious = previousloc.Col + offset.Col;
                if (colPrevious < 0)
                    colPrevious += Width;
                else if (colPrevious >= Width)
                    colPrevious -= Width;

                int colNew = newLoc.Col + offset.Col;
                if (colNew < 0)
                    colNew += Width;
                else if (colNew >= Width)
                    colNew -= Width;

                int rowPrevious = previousloc.Row + offset.Row;
                if (rowPrevious < 0)
                    rowPrevious += Height;
                else if (rowPrevious >= Height)
                    rowPrevious -= Height;

                int rowNew = newLoc.Row + offset.Row;
                if (rowNew < 0)
                    rowNew += Height;
                else if (rowNew >= Height)
                    rowNew -= Height;

                GameState.m_mapVisible[colPrevious, rowPrevious] -= 1;
                GameState.m_mapVisible[colNew, rowNew] += 1;
            }
        }

        public void AddFood(int row, int col)
        {
            map[row, col] = Tile.Food;
            FoodTiles.Add(new Location(row, col));
        }

        public void RemoveFood(int row, int col)
        {
            // an ant could move into a spot where a food just was
            // don't overwrite the space unless it is food
            if (map[row, col] == Tile.Food)
            {
                map[row, col] = Tile.Land;
            }
            FoodTiles.Remove(new Location(row, col));
        }

        public void AddWater(List<MapPoint> ListeAllWaterPoints)
        {
            List<MapPoint> ListNewWaterPt = new List<MapPoint>();
            foreach (MapPoint pt in ListeAllWaterPoints)
            {
                m_mapWater[pt.X, pt.Y] = true;
                map[pt.Y, pt.X] = Tile.Water; //TODO: raf mais bon osef.
                myPathFinder.ChangeSquareValue((ushort)pt.Y, (ushort)pt.X, 0);
            }
        }

        public void DeadAnt(int row, int col)
        {
            // food could spawn on a spot where an ant just died
            // don't overwrite the space unless it is land
            if (map[row, col] == Tile.Land)
            {
                map[row, col] = Tile.Dead;
            }

            // but always add to the dead list
            DeadTiles.Add(new Location(row, col));
        }

        public void AntHill(int row, int col, int team)
        {
            if (map[row, col] == Tile.Land)
            {
                map[row, col] = Tile.Hill;
            }

            AntHill hill = new AntHill(row, col, team);
            if (team == 0)
                MyHills.Add(hill);
            else
                EnemyHills.Add(hill);
        }
        #endregion

        /// <summary>
        /// Gets whether <paramref name="location"/> is passable or not.
        /// </summary>
        /// <param name="location">The location to check.</param>
        /// <returns><c>true</c> if the location is not water, <c>false</c> otherwise.</returns>
        /// <seealso cref="GetIsUnoccupied"/>
        public bool GetIsPassable(Location location)
        {
            return map[location.Row, location.Col] != Tile.Water;
        }

        /// <summary>
        /// Gets whether <paramref name="location"/> is occupied or not.
        /// </summary>
        /// <param name="location">The location to check.</param>
        /// <returns><c>true</c> if the location is passable and does not contain an ant, <c>false</c> otherwise.</returns>
        public bool GetIsUnoccupied(Location location)
        {
            return GetIsPassable(location) && map[location.Row, location.Col] != Tile.Ant;
        }

		public bool GetIsNotFood(Location location)
		{
			return map[location.Row, location.Col] != Tile.Food;
		}

        /// <summary>
        /// Gets the destination if an ant at <paramref name="location"/> goes in <paramref name="direction"/>, accounting for wrap around.
        /// </summary>
        /// <param name="location">The starting location.</param>
        /// <param name="direction">The direction to move.</param>
        /// <returns>The new location, accounting for wrap around.</returns>
        public Location GetDestination(Location location, Direction direction)
        {
            Location delta = Ants.Aim[direction];

            int row = (location.Row + delta.Row) % Height;
            if (row < 0) row += Height; // because the modulo of a negative number is negative

            int col = (location.Col + delta.Col) % Width;
            if (col < 0) col += Width;

            return new Location(row, col);
        }

        /// <summary>
        /// Gets the distance between <paramref name="loc1"/> and <paramref name="loc2"/>.
        /// </summary>
        /// <param name="loc1">The first location to measure with.</param>
        /// <param name="loc2">The second location to measure with.</param>
        /// <returns>The distance between <paramref name="loc1"/> and <paramref name="loc2"/></returns>
        public int GetDistance(Location loc1, Location loc2, ref bool bTimeout, ref List<AStarAlgo.MapPoint> ListPoint)
        {
            bTimeout = false;
            if (GameState.IsTimeOutCompute())
                return 10000;

            Route route = new Route(loc1, loc2, 0);
            ListPoint = MyBot.FindWay(loc1, loc2);
            if (ListPoint != null)
            {
                return ListPoint.Count;
            }
            else
            {
                #region log
#if LOG_ENABLED
                MyBot.Log(string.Format("TIMEOUT pr ({0},{1}) vers ({2},{3})", loc1.Col, loc1.Row, loc2.Col, loc2.Row), 19);
#endif
                #endregion

                bTimeout = true;
                return 10000;
            }
        }
			
        public int GetAirDistanceBetweenLocation(Location LocA, Location LocB, ref int colDistanceC, ref int rowDistanceC)
        {
            int rowDistance = 0;
            if (LocA.Row > LocB.Row)
            {
                int distance3 = LocB.Row + Height - LocA.Row;
                int distance4 = LocA.Row - LocB.Row;
                rowDistance = distance3 < distance4 ? distance3 : distance4;
            }
            else if (LocA.Row < LocB.Row)
            {
                int distance3 = LocA.Row + Height - LocB.Row;
                int distance4 = LocB.Row - LocA.Row;
                rowDistance = distance3 < distance4 ? distance3 : distance4;
            }

            int colDistance = 0;
            if (LocA.Col > LocB.Col)
            {
                int distance3 = LocB.Col + Width - LocA.Col;
                int distance4 = LocA.Col - LocB.Col;
                colDistance = distance3 < distance4 ? distance3 : distance4;
            }
            else if (LocA.Col < LocB.Col)
            {
                int distance3 = LocA.Col + Width - LocB.Col;
                int distance4 = LocB.Col - LocA.Col;
                colDistance = distance3 < distance4 ? distance3 : distance4;
            }

            colDistanceC = colDistance;
            rowDistanceC = rowDistance;
            return rowDistance + colDistance;
        }


        public bool GetIsCloseEnoughFromTheAnt(Ant ant, Location loc, MyBot.eMovementType type, int distanceOver=0)
        {
            int colD = -1;
            int rowD = -1;
            int distance = GetAirDistanceBetweenLocation(ant, loc, ref colD, ref rowD);

            int distanceAllowed = -1;
            if (distanceOver != 0)
                distanceAllowed = distanceOver;
            else
            {
                switch (type)
                {
                    case MyBot.eMovementType.HillPatrol:
                        distanceAllowed = 20;
                        break;
                    case MyBot.eMovementType.attackRange:
                        distanceAllowed = 20;
                        break;
                    case MyBot.eMovementType.CloseFood:
                        distanceAllowed = 4;
                        break;
                    case MyBot.eMovementType.AttackHills:
                        distanceAllowed = MyBot.g_iDistanceAllowed + 30;
                        break;
                    case MyBot.eMovementType.DefendHill:
                        distanceAllowed = MyBot.g_iDistanceAllowed + 5;
                        break;
                    case MyBot.eMovementType.Food:
                        distanceAllowed = MyBot.g_iDistanceAllowed + 5;
                        break;
                    case MyBot.eMovementType.Explore:
                        distanceAllowed = MyBot.g_iDistanceAllowed;
                        break;
                }
            }
            if (distance < distanceAllowed)
                return true;
            else
                return false;
        }

        public bool GetIsVisible(Location loc)
        {
            foreach (Ant ant in this.MyAnts)
            {
                foreach (Location offset in g_VisibleOffsets)
                {
                    int colMaybe = ant.Col + offset.Col;
                    if (colMaybe < 0)
                        colMaybe += Width;
                    else if (colMaybe >= Width)
                        colMaybe -= Width;

                    int rowMaybe = ant.Row + offset.Row;
                    if (rowMaybe < 0)
                        rowMaybe += Height;
                    else if (rowMaybe >= Height)
                        rowMaybe -= Height;

                    if (colMaybe == loc.Col &&
                            rowMaybe == loc.Row)
                    {
                        return true;
                    }
                }
            }
            return false;
        }


        public bool IsLocInViewRangeFromOneOfMyHill(Location antLoc)
        {
            foreach (AntHill hill in MyHills)
            {
                int colD = -1;
                int rowD = -1;
                int distanceB = GetAirDistanceBetweenLocation(antLoc, hill, ref colD, ref rowD);
                int square = colD * colD + rowD * rowD;
                if (square < ViewRadius2)
                {
                    return true;
                }
            }
            return false;
        }

        public static void UpdateFightingMapForMyAnt(Ant ant)
        {
            foreach (Location offset in g_AttackoffsetsStrict) 
            {
                int colMaybe = ant.Col + offset.Col;
                if (colMaybe < 0)
                    colMaybe += Width;
                else if (colMaybe >= Width)
                    colMaybe -= Width;

                int rowMaybe = ant.Row + offset.Row;
                if (rowMaybe < 0)
                    rowMaybe += Height;
                else if (rowMaybe >= Height)
                    rowMaybe -= Height;

                m_fightTotalArray[colMaybe, rowMaybe]++;
                m_fightInfluenceArray[0, colMaybe, rowMaybe]++;
            }
        }

        public enum eFightingStatus { SAFE = 0, KILL = 1, DIE = 2 };
        public void UpdateFightingMap()
        {

            m_fightTotalArray = new int[Width, Height];
            m_fightInfluenceArray = new int[g_EnemyPlayerNumber+1, Width, Height];
            m_fightFightingArray = new int[g_EnemyPlayerNumber+1, Width, Height];
            m_fightStatusArray = new eFightingStatus[g_EnemyPlayerNumber+1, Width, Height];

            foreach (Ant ant in MyAnts)
            {
                UpdateFightingMapForMyAnt(ant);
                //tester les 4 points.
                //Top:
                //s
            }


            foreach (Ant ant in EnemyAnts)
            {
                //g_AttackoffsetsStrict
                foreach (Location offset in g_Attackoffsets)
                {                    
                    int colMaybe = ant.Col + offset.Col;
                    if (colMaybe < 0)
                        colMaybe += Width;
                    else if (colMaybe >= Width)
                        colMaybe -= Width;

                    int rowMaybe = ant.Row + offset.Row;
                    if (rowMaybe < 0)
                        rowMaybe += Height;
                    else if (rowMaybe >= Height)
                        rowMaybe -= Height;

                    m_fightTotalArray[colMaybe, rowMaybe]++;
                    m_fightInfluenceArray[ant.Team, colMaybe, rowMaybe]++;
                }             
            }


            for (int i = 0; i < Width; i++)
            {
                for (int j = 0; j < Height; j++)
                {
                    for (int player = 1; player < g_EnemyPlayerNumber+1; player++)
                    {
                        m_fightFightingArray[0, i, j] += m_fightInfluenceArray[player, i, j];
                    }
                }
            }


            foreach (Ant ant in MyAnts)
            {
                int colRight = ant.Col + 1;
                if (colRight >= Width)
                    colRight -= Width;

                int colLeft = ant.Col - 1;
                if (colLeft < 0)
                    colLeft += Width;

                int rowBot = ant.Row + 1;
                if (rowBot >= Height)
                    rowBot -= Height;

                int rowTop = ant.Row - 1;
                if (rowTop < 0)
                    rowTop += Height;


                //todo : on peut zapper la flotte ici... plus tard.
                //5cas:
                //top
                int maxValueTop = 0;
                int maxValueBot = 0;
                int maxValueLeft = 0;
                int maxValueRight = 0;
                int maxValueCurrent = 0;

                maxValueTop += m_fightFightingArray[0, ant.Col, rowTop];
                maxValueBot += m_fightFightingArray[0, ant.Col, rowBot];
                maxValueLeft += m_fightFightingArray[0, colLeft, ant.Row];
                maxValueRight += m_fightFightingArray[0, colRight, ant.Row];
                maxValueCurrent += m_fightFightingArray[0, ant.Col, ant.Row];

                //Top
                if (m_fightInfluenceArray[0, ant.Col, rowTop] > maxValueTop)
                    m_fightStatusArray[0, ant.Col, rowTop] = eFightingStatus.SAFE;
                else if (m_fightInfluenceArray[0, ant.Col, rowTop] == maxValueTop)
                    m_fightStatusArray[0, ant.Col, rowTop] = eFightingStatus.KILL;
                else
                    m_fightStatusArray[0, ant.Col, rowTop] = eFightingStatus.DIE;

                //Bot
                if (m_fightInfluenceArray[0, ant.Col, rowBot] > maxValueBot)
                    m_fightStatusArray[0, ant.Col, rowBot] = eFightingStatus.SAFE;
                else if (m_fightInfluenceArray[0, ant.Col, rowBot] == maxValueBot)
                    m_fightStatusArray[0, ant.Col, rowBot] = eFightingStatus.KILL;
                else
                    m_fightStatusArray[0, ant.Col, rowBot] = eFightingStatus.DIE;

                //Left
                if (m_fightInfluenceArray[0, colLeft, ant.Row] > maxValueLeft)
                    m_fightStatusArray[0, colLeft, ant.Row] = eFightingStatus.SAFE;
                else if (m_fightInfluenceArray[0, colLeft, ant.Row] == maxValueLeft)
                    m_fightStatusArray[0, colLeft, ant.Row] = eFightingStatus.KILL;
                else
                    m_fightStatusArray[0, colLeft, ant.Row] = eFightingStatus.DIE;

                //Right
                if (m_fightInfluenceArray[0, colRight, ant.Row] > maxValueRight)
                    m_fightStatusArray[0, colRight, ant.Row] = eFightingStatus.SAFE;
                else if (m_fightInfluenceArray[0, colRight, ant.Row] == maxValueRight)
                    m_fightStatusArray[0, colRight, ant.Row] = eFightingStatus.KILL;
                else
                    m_fightStatusArray[0, colRight, ant.Row] = eFightingStatus.DIE;


                //NotMove
                if (m_fightInfluenceArray[0, ant.Col, ant.Row] > maxValueCurrent)
                    m_fightStatusArray[0, ant.Col, ant.Row] = eFightingStatus.SAFE;
                else if (m_fightInfluenceArray[0, ant.Col, ant.Row] == maxValueCurrent)
                    m_fightStatusArray[0, ant.Col, ant.Row] = eFightingStatus.KILL;
                else
                    m_fightStatusArray[0, ant.Col, ant.Row] = eFightingStatus.DIE;

            }    
        }       

        public static void FindDirections(Location antLoc, List<MapPoint> listPoints, ref Direction dPlanA, ref Direction dPlanB, ref bool bPlanBFound)
        {
            MapPoint point = listPoints[listPoints.Count - 2];    

            #region log
#if LOG_ENABLED
                    Log(string.Format("nombre de point pour y aller: {2} doMoveLocation au point ({0},{1}))", point.X, point.Y, listPoints.Count), 0);
#endif
            #endregion

            if ((point.X == 0 && antLoc.Col == Width - 1) || (antLoc.Col == 0 && point.X == Width - 1) || (point.Y == 0 && antLoc.Row == Height - 1) || (antLoc.Row == 0 && point.Y == Height - 1))
            {
                if (point.X == 0)
                    dPlanA = Direction.East;

                if (point.X == Width - 1)
                    dPlanA = Direction.West;

                if (point.Y == 0)
                    dPlanA = Direction.South;

                if (point.Y == Height - 1)
                    dPlanA = Direction.North;
            }
            else
            {
                if (point.X > antLoc.Col)
                    dPlanA = Direction.East;
                else if (point.X < antLoc.Col)
                    dPlanA = Direction.West;

                if (point.Y > antLoc.Row)
                    dPlanA = Direction.South;
                else if (point.Y < antLoc.Row)
                    dPlanA = Direction.North;

            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="enemyAnt"></param>
        /// <param name="myAntHillSpoted"></param>
        /// <returns></returns>
        public bool IsEnemyAntCloseFromMyHill(Ant enemyAnt, ref AntHill myAntHillSpoted)
        {
            bool bFound = false;
            int distance = 10000;
            foreach (AntHill hill in MyBot.m_ListMyHills)
            {
                int colD = -1;
                int rowD = -1;
                int distanceTMP = GetAirDistanceBetweenLocation(enemyAnt, hill, ref colD, ref rowD);

                if (distanceTMP < 20)
                {
                    bool bTimeout = false;
                    List<MapPoint> listPoint = new List<MapPoint>();
                    GameState.myPathFinder.ChangeSquareValue((ushort)hill.Row, (ushort)hill.Col, 1);
                    int distanceReal = GetDistance(enemyAnt, hill, ref bTimeout, ref listPoint);
                    GameState.myPathFinder.ChangeSquareValue((ushort)hill.Row, (ushort)hill.Col, 0);
                    if (distanceReal < 20 && distanceTMP < distance) 
                    {
                        distance = distanceReal;
                        myAntHillSpoted = hill;
                        bFound = true;
                    }
                }
            }
            return bFound;
        }

        public List<Ant> GetEnemyListOrderFromDistance(Ant ant)
        {
            List<Ant> ListEnemy = new List<Ant>();
            List<Route> ListRoute = new List<Route>();
            foreach (Ant enemyAnt in EnemyAnts)
            {
                if (GameState.IsTimeOutGlobal())
                    return ListEnemy;

                int colD = -1;
                int rowD = -1;
                int distance = GetAirDistanceBetweenLocation(ant, enemyAnt, ref colD, ref rowD);
                ListRoute.Add(new Route(ant, enemyAnt, distance));

            }

            ListRoute.Sort();
            foreach (Route route in ListRoute)
            {
                ListEnemy.Add(new Ant(route.end.Row, route.end.Col, 1));
            }
            return ListEnemy;
        }
    }
}

