using System;
using System.Collections.Generic;

namespace Ants {

	public class Location : IEquatable<Location> {

		/// <summary>
		/// Gets the row of this location.
		/// </summary>
		public int Row { get; set; }

		/// <summary>
		/// Gets the column of this location.
		/// </summary>
		public int Col { get; set; }

        public Location()
        {
        }

        public void AddColRow(int col, int row)
        {
            Col = Col + col;
						while (Col >= GameState.Width)
							Col -= GameState.Width;
						while (Col <0)
                Col += GameState.Width;

            Row = Row + row;
						while (Row >= GameState.Height)
                Row -= GameState.Height;
						while (Row < 0)
                Row += GameState.Height;
        }

		public Location (int row, int col) {
			this.Row = row;
			this.Col = col;
		}

		public override bool Equals (object obj) {
			if (ReferenceEquals (null, obj))
				return false;
			if (ReferenceEquals (this, obj))
				return true;
			if (obj.GetType() != typeof (Location))
				return false;

			return Equals ((Location) obj);
		}

		public bool Equals (Location other) {
			if (ReferenceEquals (null, other))
				return false;
			if (ReferenceEquals (this, other))
				return true;

			return other.Row == this.Row && other.Col == this.Col;
		}

        public bool Equals(Ant other)
        {
            return other.Row == this.Row && other.Col == this.Col;
        }



		public override int GetHashCode()
		{
			unchecked {
				return (this.Row * 397) ^ this.Col;
			}
		}
	}

    public class VisibleLocation : Location, IEquatable<VisibleLocation>, IComparable
    {
        public int Visibility { get; set; }

        public VisibleLocation(int col, int row, int visibility)
        {
            this.Col = col;
            this.Row = row;
            this.Visibility = visibility;
        }

        public bool Equals(VisibleLocation other)
        {
            return other.Visibility == this.Visibility;
        }

        public override int GetHashCode()
        {
            unchecked
            {
                int result = this.Col;
                result = (result * 397) ^ this.Row + Visibility + this.Col *50;
                return result;
            }
        }

        public int CompareTo(VisibleLocation other)
        {
            int visibleCompare = this.Visibility - other.Visibility;
            return visibleCompare;
        }

        public int CompareTo(Object o)
        {
            int result = -1;
            if (typeof(VisibleLocation) == o.GetType())
            {
                VisibleLocation route = (VisibleLocation)o;
                result = this.CompareTo(route);
            }
            return result;
        }
    }

	public class TeamLocation : Location, IEquatable<TeamLocation> {
		/// <summary>
		/// Gets the team of this ant.
		/// </summary>
		public int Team { get; private set; }

		public TeamLocation (int row, int col, int team) : base (row, col) {
			this.Team = team;
		}

		public bool Equals(TeamLocation other) {
			return base.Equals (other) && other.Team == Team;
		}

		public override int GetHashCode()
		{
			unchecked {
				int result = this.Col;
				result = (result * 397) ^ this.Row;
				result = (result * 397) ^ this.Team;
				return result;
			}
		}
	}
	
	public class Ant : TeamLocation, IEquatable<Ant> {
		public Ant (int row, int col, int team) : base (row, col, team) {
		}

		public bool Equals (Ant other) {
            return (other.Col == Col && other.Row == Row);
		}

        public bool Equals(AntHill other) {
            return base.Equals(other);
        }

		public Ant Clone()
		{
			return new Ant(Row, Col, Team);
		}
	}

	public class AntHill : TeamLocation, IEquatable<AntHill> {
		public AntHill (int row, int col, int team) : base (row, col, team) {
		}

		public bool Equals (AntHill other) {
            return (other.Col == Col && other.Row == Row);
		}


	}
}

